#include "ofApp.h"

//--------------------------------------------------------------
void ofApp::setup(){	
	counter = 0;
	ofSetCircleResolution(100);
	texScreen.allocate(320,240,GL_RGB);
	ofBackground(230,230,240);
    
    ///
    
    camWidth = 320;  // try to grab at this size.
    camHeight = 240;
    
    //we can now get back a list of devices.
    vector<ofVideoDevice> devices = vidGrabber.listDevices();
    
    for(int i = 0; i < devices.size(); i++){
        if(devices[i].bAvailable){
            ofLogNotice() << devices[i].id << ": " << devices[i].deviceName;
        }else{
            ofLogNotice() << devices[i].id << ": " << devices[i].deviceName << " - unavailable ";
        }
    }
    
    vidGrabber.setDeviceID(0);
    vidGrabber.setDesiredFrameRate(60);
    vidGrabber.initGrabber(camWidth, camHeight);
    
    videoInverted.allocate(camWidth, camHeight, OF_PIXELS_RGB);
    videoTexture.allocate(videoInverted);
    ofSetVerticalSync(true);
//ofPixels & texPixels;
}
 

//--------------------------------------------------------------
void ofApp::update(){
	counter += 1;
    
    ///
    
    ofBackground(100, 100, 100);
    vidGrabber.update();
    
    if(vidGrabber.isFrameNew()){
        ofPixels & pixels = vidGrabber.getPixels();
        
        texScreen.readToPixels(texPixels);
        for(int i = 0; i < pixels.size(); i++){
            
            if(texPixels[i]==255){
                videoInverted[i] = pixels[i];
            }else{
                videoInverted[i]=127;
            }
        }
        videoTexture.loadData(videoInverted);
    }

}


//--------------------------------------------------------------
void ofApp::draw(){

	// 1st, draw on screen:
	ofSetHexColor(0x000000);
	ofDrawRectangle(100,100,320,240);
	
	ofSetHexColor(0xffffff);
	ofPushMatrix();
		ofTranslate(200,200,0);
		ofRotate(counter,0,0,1);
		ofDrawCircle(0,0,80);
		ofDrawCircle(100,0,10);	// a small one
	ofPopMatrix();
	ofSetHexColor(0x000000);
	ofDrawBitmapString("(a) on screen", 150,200);

	ofSetHexColor(0xFFFFFF);
	ofDrawCircle(mouseX, mouseY,20);
	

	// 2nd, grab a portion of the screen into a texture
	// this is quicker then grabbing into an ofImage
	// because the transfer is done in the graphics card
	// as opposed to bringing pixels back to memory
	// note: you need to allocate the texture to the right size
	texScreen.loadScreenData(100,100,320,240);
	
	

	// finally, draw that texture on screen, how ever you want
	// (note: you can even draw the texture before you call loadScreenData, 
	// in order to make some trails or feedback type effects)
	ofPushMatrix();
		ofSetHexColor(0xffffff);
		ofTranslate(550,100,0);
		//glRotatef(counter, 0.1f, 0.03f, 0);
		float width = 320;
		float height = 240;
		texScreen.draw(0,0,width,height);
	ofPopMatrix();

//	ofPushMatrix();
//		ofSetHexColor(0xffffff);
//		ofTranslate(700,210,0);
//		ofRotate(counter, 0.1f, 0.03f, 0);
//		texScreen.draw(-50,-50,100,100);
//	ofPopMatrix();

	ofSetHexColor(0x333333);
	ofDrawBitmapString("(b) in a texture, very meta!", 500,200);
    
    ///
    
    ofSetHexColor(0xffffff);
    vidGrabber.draw(100, 500);
    videoTexture.draw(550, 500, camWidth, camHeight);

    
}

//--------------------------------------------------------------
void ofApp::keyPressed  (int key){ 
	
}

//--------------------------------------------------------------
void ofApp::keyReleased(int key){ 
	
}

//--------------------------------------------------------------
void ofApp::mouseMoved(int x, int y ){
	
}

//--------------------------------------------------------------
void ofApp::mouseDragged(int x, int y, int button){
	
}

//--------------------------------------------------------------
void ofApp::mousePressed(int x, int y, int button){
	
}

//--------------------------------------------------------------
void ofApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mouseEntered(int x, int y){

}

//--------------------------------------------------------------
void ofApp::mouseExited(int x, int y){

}

//--------------------------------------------------------------
void ofApp::windowResized(int w, int h){

}

//--------------------------------------------------------------
void ofApp::gotMessage(ofMessage msg){

}

//--------------------------------------------------------------
void ofApp::dragEvent(ofDragInfo dragInfo){ 

}
